#include "stdafx.h"
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
//
// BEGIN cmdDES_Test command
//

#pragma region cmdDES_Test command

class CCommandcmdDES_Test : public CRhinoCommand
{
public:
	CCommandcmdDES_Test() = default;
	UUID CommandUUID() override
	{
    // {BA624CD2-64ED-4DEE-A64A-D5DFC44817C4}
    static const GUID cmdDES_TestCommand_UUID =
    { 0xba624cd2, 0x64ed, 0x4dee,{ 0xa6, 0x4a, 0xd5, 0xdf, 0xc4, 0x48, 0x17, 0xc4 } };
		return cmdDES_TestCommand_UUID;
	}
	const wchar_t* EnglishCommandName() override { return L"DES_TextureTest"; }
	CRhinoCommand::result RunCommand(const CRhinoCommandContext& context) override;
};

// The one and only CCommandcmdDES_Test object
static class CCommandcmdDES_Test thecmdDES_TestCommand;

bool CreateDefaultTextureMapping(const CRhinoObject& obj, const int channelId, const UUID& pluginUuid, UUID& uuidOut)
{
  CRhinoDoc* pDoc = obj.Document();
  if (NULL == pDoc)
    return false;

  ON_TextureMapping tm;
  tm.SetSurfaceParameterMapping();

  const int index = pDoc->m_texture_mapping_table.AddTextureMapping(tm);
  if (index < 0)
    return false;

  const UUID uuidMapping = pDoc->m_texture_mapping_table[index].Id();

  ON_3dmObjectAttributes attributes = obj.Attributes();

  if (!attributes.m_rendering_attributes.AddMappingChannel(pluginUuid, channelId, uuidMapping))
  {
    if (!attributes.m_rendering_attributes.DeleteMappingChannel(pluginUuid, channelId))
      return false;

    if (!attributes.m_rendering_attributes.AddMappingChannel(pluginUuid, channelId, uuidMapping))
      return false;
  }

  if (!pDoc->ModifyObjectAttributes(CRhinoObjRef(&obj), attributes, true))
    return false;

  uuidOut = uuidMapping;

  return true;
}

const ON_MappingChannel* GetTextureMappingChannel(const CRhinoObject* pObject, const int channelId, const UUID & pluginUuid)
{
  if (NULL == pObject)
    return NULL;

  const ON_MappingRef* pMapRef = pObject->Attributes().m_rendering_attributes.MappingRef(pluginUuid);
  if (NULL == pMapRef)
    return NULL;

  for (int i = 0; i < pMapRef->m_mapping_channels.Count(); i++)
  {
    if (pMapRef->m_mapping_channels[i].m_mapping_channel_id == channelId)
      return &(pMapRef->m_mapping_channels[i]);
  }

  return NULL;
}

bool SetCustomMeshMapping(const CRhinoObject& object, CRhinoDoc& doc, ON_Mesh* pCustomMappingMesh)
{
  ON_UUID uuidRenderPlugIn = RhinoApp().GetDefaultRenderApp();
  const ON_MappingChannel* pMappingChannel = GetTextureMappingChannel(&object, 1, uuidRenderPlugIn);
  if (nullptr == pMappingChannel)
  {
    UUID uuidMapping;
    CreateDefaultTextureMapping(object, 1, uuidRenderPlugIn, uuidMapping);
    pMappingChannel = GetTextureMappingChannel(&object, 1, uuidRenderPlugIn);
  }

  const int iTextureMapping = doc.m_texture_mapping_table.FindTextureMapping(pMappingChannel);
  if (iTextureMapping < 0 || iTextureMapping >= doc.m_texture_mapping_table.TextureMappingCount())
  {
    delete pCustomMappingMesh;
    return false;
  }

  ON_TextureMapping textureMapping = doc.m_texture_mapping_table[iTextureMapping];

  textureMapping.m_Pxyz = ON_Xform::IdentityTransformation;
  textureMapping.m_Nxyz = ON_Xform::IdentityTransformation;
  textureMapping.m_uvw = ON_Xform::IdentityTransformation;

  textureMapping.m_projection = ON_TextureMapping::PROJECTION::clspt_projection;
  textureMapping.m_type = ON_TextureMapping::TYPE::mesh_mapping_primitive;

  pCustomMappingMesh->Transform(pMappingChannel->m_object_xform.Inverse());

  textureMapping.SetCustomMappingPrimitive(pCustomMappingMesh);

  if (!doc.m_texture_mapping_table.ModifyTextureMapping(textureMapping, iTextureMapping))
  {
    delete pCustomMappingMesh;
    return false;
  }

  const ON_Xform * pObjectXform = (pMappingChannel == nullptr ? nullptr : &pMappingChannel->m_object_xform);
  object.SetTextureCoordinates(textureMapping, pObjectXform, false);

  return true;
}

CRhinoCommand::result CCommandcmdDES_Test::RunCommand(const CRhinoCommandContext& context)
{
  //Set Rendered display mode
  CRhinoViewport& vp = ::RhinoApp().ActiveView()->ActiveViewport();
  if (vp.View().m_display_mode_id != ON_StandardDisplayModeId::Rendered)
  {
    vp.SetDisplayMode(ON_StandardDisplayModeId::Rendered);
    ::RhinoApp().ActiveDoc()->Redraw(CRhinoView::regenerate_display_hint);
  }

  CRhinoGetObject get;
  get.SetGeometryFilter(CRhinoObject::surface_object);
  get.AcceptNothing(true);
  get.SetCommandPrompt(L"Select Surface to Texture Mapping");

  CRhinoObject* pObject = NULL;
  {
    for (;;)
    {
      context.m_doc.UnselectAll();
      context.m_doc.Redraw();

      CRhinoGet::result res = get.GetObjects(1, 1);
      if (res == CRhinoGet::object)
      {
        pObject = const_cast<CRhinoObject*>(get.Object(0).Object());
        break;
      }
      else if (res == CRhinoGet::cancel)
        return CRhinoCommand::cancel;

    }
  }
  double ScaleX = 5., ScaleY = 5.;
  {
    CRhinoGetNumber gn_x;
    gn_x.SetCommandPrompt(L"Input ScaleX");
    gn_x.AcceptNothing();
    gn_x.GetNumber();
    if (gn_x.CommandResult() != CRhinoCommand::success)
      return gn_x.CommandResult();
    ScaleX = gn_x.Number();

    CRhinoGetNumber gn_y;
    gn_y.SetCommandPrompt(L"Input ScaleY");
    gn_y.AcceptNothing();
    gn_y.GetNumber();
    if (gn_y.CommandResult() != CRhinoCommand::success)
      return gn_y.CommandResult();
    ScaleY = gn_y.Number();
  }

  if (pObject)
  {
    CString m_BmpFile;
    CRhinoGetFileDialog GetFileDlg;
    GetFileDlg.DisplayFileDialog(CRhinoGetFileDialog::open_bitmap_dialog, ON_wString(m_BmpFile));
    m_BmpFile = GetFileDlg.FileName();

    //Texture mapping
    if (m_BmpFile != L"")
    {
      //Texture mapping
      if (m_BmpFile != L"")
      {
        const CRhinoObjectAttributes&  A = pObject->Attributes();
        const ON_Material Material = pObject->ObjectMaterial();
        CRhinoObjectAttributes A2(A);
        ON_Material M(Material);
        M.DeleteTexture(NULL, ON_Texture::TYPE::bitmap_texture);

        ON_Texture tex;
        tex.m_image_file_reference.SetFullPath(m_BmpFile, false);
        tex.m_bOn = true;
        tex.m_type = ON_Texture::TYPE::bitmap_texture;
        tex.m_mode = ON_Texture::MODE::decal_texture;
        int material_index = M.AddTexture(tex);
        if (A2.m_material_index == -1)
          A2.m_material_index = material_index;

        pObject->ModifyAttributes(A2);
        CRhinoMaterialTable& material_table = ::RhinoApp().ActiveDoc()->m_material_table;
        material_table.ModifyMaterial(M, A2.m_material_index, false);
        context.m_doc.Regen();

        //const CRhinoObjRef& object_ref(pObject);
        //context.m_doc.ModifyObjectAttributes(object_ref, A2);
        //context.m_doc.Regen();

        const ON_Brep* Brep = ON_Brep::Cast(pObject->Geometry());
        ON_BrepFace* F = Brep->Face(0);
        ON_Interval u_dom, v_dom;
        const ON_Surface* Sur = *Brep->m_S.At(0);
        {
          u_dom = Sur->Domain(0);
          v_dom = Sur->Domain(1);

          ON_Curve* uCrv = Sur->IsoCurve(0, v_dom.Mid());
          ON_Curve* vCrv = Sur->IsoCurve(1, u_dom.Mid());

          double len0, len1;
          uCrv->GetLength(&len0);
          vCrv->GetLength(&len1);
          delete uCrv;	delete vCrv;
        }

        // Don't modify render mesh itself. Modify duplicate of it instead.
        ON_Mesh* mappingMesh = F->Mesh(ON::render_mesh)->Duplicate();
         
        ON_2fPointArray Texs = mappingMesh->m_T;
        for (int i = 0; i < mappingMesh->m_T.Count(); i++)
        {
          const ON_2dPoint* p2d = mappingMesh->m_S.At(i);
          ON_2dPoint Pnt; Pnt.x = p2d->x; Pnt.y = p2d->y;

          double x = Pnt.x;
          double y = Pnt.y;

          double tx = (x - u_dom[0]) / u_dom.Length();
          double ty = (y - v_dom[0]) / v_dom.Length();

          tx /= ScaleX;
          ty /= ScaleY;
          ON_2fPoint* p2f = Texs.At(i);
          p2f->x = float(tx);
          p2f->y = float(ty);
        }
        mappingMesh->m_T = Texs;

        //Move Vertex positoin
        ON_3fPointArray Vertex = mappingMesh->m_V;
        for (int i = 0; i < Vertex.Count(); i++)
        {
          ON_3fPoint* p3f = Vertex.At(i);
          ON_3dVector vec = ON_zaxis;
          vec.Unitize(); vec = vec * 1.5;
          p3f->z = p3f->z+float(vec.z);
        }
        mappingMesh->m_V = Vertex;
        // Set the modified mesh as mesh mapping primitive
        SetCustomMeshMapping(*pObject, *context.Document(), mappingMesh);
      }
    }
  }
  ::RhinoApp().ActiveDoc()->Regen();
  return CRhinoCommand::success;
}


#pragma endregion

//
// END cmdDES_Test command
//
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
//
// BEGIN cmdDES_MeshMove command
//

#pragma region cmdDES_MeshMove command

class CCommandcmdDES_MeshMove : public CRhinoCommand
{
public:
  CCommandcmdDES_MeshMove() = default;
  UUID CommandUUID() override
  {
    // {778B37A7-A7C5-46AA-8806-9717A729A652}
    static const GUID cmdDES_MeshMoveCommand_UUID =
    { 0x778b37a7, 0xa7c5, 0x46aa,{ 0x88, 0x6, 0x97, 0x17, 0xa7, 0x29, 0xa6, 0x52 } };
    return cmdDES_MeshMoveCommand_UUID;
  }
  const wchar_t* EnglishCommandName() override { return L"DES_MeshMove"; }
  CRhinoCommand::result RunCommand(const CRhinoCommandContext& context) override;
};

// The one and only CCommandcmdDES_MeshMove object
static class CCommandcmdDES_MeshMove thecmdDES_MeshMoveCommand;

CRhinoCommand::result CCommandcmdDES_MeshMove::RunCommand(const CRhinoCommandContext& context)
{
  //Set Rendered display mode
  CRhinoViewport& vp = ::RhinoApp().ActiveView()->ActiveViewport();
  if (vp.View().m_display_mode_id != ON_StandardDisplayModeId::Rendered)
  {
    vp.SetDisplayMode(ON_StandardDisplayModeId::Rendered);
    ::RhinoApp().ActiveDoc()->Redraw(CRhinoView::regenerate_display_hint);
  }

  CRhinoGetObject get;
  get.SetGeometryFilter(CRhinoObject::surface_object);
  get.AcceptNothing(true);
  get.SetCommandPrompt(L"Select Surface to move mesh");

  CRhinoObject* pObject = NULL;
  {
    for (;;)
    {
      context.m_doc.UnselectAll();
      context.m_doc.Redraw();

      CRhinoGet::result res = get.GetObjects(1, 1);
      if (res == CRhinoGet::object)
      {
        pObject = const_cast<CRhinoObject*>(get.Object(0).Object());
        break;
      }
      else if (res == CRhinoGet::cancel)
        return CRhinoCommand::cancel;

    }
  }
  if (pObject)
  {
    // Don't modify render mesh itself. Modify duplicate of it instead.
    const ON_Brep* Brep = ON_Brep::Cast(pObject->Geometry());
    ON_BrepFace* F = Brep->Face(0);
    ON_Mesh* mappingMesh = F->Mesh(ON::render_mesh)->Duplicate();

    //Move Vertex positoin
    ON_3fPointArray Vertex = mappingMesh->m_V;
    for (int i = 0; i < Vertex.Count(); i++)
    {
      ON_3fPoint* p3f = Vertex.At(i);
      ON_3dVector vec = ON_zaxis;
      vec.Unitize(); vec = vec * 1.5;
      p3f->z = p3f->z + float(vec.z);
    }
    mappingMesh->m_V = Vertex;
    // Set the modified mesh as mesh mapping primitive
    SetCustomMeshMapping(*pObject, *context.Document(), mappingMesh);

  }
  ::RhinoApp().ActiveDoc()->Regen();
  return CRhinoCommand::success;
}

#pragma endregion

//
// END cmdDES_MeshMove command
//
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
